#include "common.h"

#include "ximc.h"
#include "util.h"
#include "metadata.h"
#include "platform.h"
#include "protosup.h"
#include "loader.h"
#include "ximc-gen.h"

#if !defined(LIBXIMC_EXPORTS)
#error LIBXIMC_EXPORTS must not be defined when building a library
#endif

#if defined(__cplusplus)
extern "C" {
#endif

/* @@GENERATED_CODE@@ */

#if defined(__cplusplus)
};
#endif

// vim: syntax=c tabstop=4 shiftwidth=4

